@@ -67,7 +67,8 @@ accessible_object_finalize (GObject *object)
         Accessible *accessible = ACCESSIBLE (object);
 
         printf("accessible_object_finalize called\n");
-        g_object_unref (accessible->atko);
+	ATK_OBJECT (accessible->atko); /* assertion */
+        g_object_unref (G_OBJECT(accessible->atko));
         accessible->atko = NULL;
 
         printf("atko freed, calling parent finalize\n");
@@ -88,7 +89,7 @@ impl_accessibility_accessible_get_name (PortableServer_Servant servant,
     retval = CORBA_string_dup (retval);
   else
     retval = CORBA_string_dup ("");
-  fprintf (stderr, "Accessible get_name called: %s\n", retval);
+
   return retval;
 }
 
@@ -115,7 +116,7 @@ impl_accessibility_accessible_get_description (PortableServer_Servant servant,
   CORBA_char * retval;
   Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
   retval = CORBA_string_dup (atk_object_get_description (accessible->atko));
-  fprintf (stderr, "Accessible get_description called: %s\n", retval);
+
   return retval;
 }
 
@@ -143,9 +144,9 @@ impl_accessibility_accessible_get_parent (PortableServer_Servant servant,
   Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
   AtkObject *parent;
   parent = atk_object_get_parent (accessible->atko);
-  retval = bonobo_object_corba_objref (bonobo_object (accessible_new (parent)));
+  retval = BONOBO_OBJREF (accessible_new (parent));
   printf ("Accessible get_parent called\n");
-  return retval;
+  return CORBA_Object_duplicate (retval, &ev);
 }
 
 /*
@@ -187,8 +188,43 @@ impl_accessibility_accessible_get_child_at_index (PortableServer_Servant servant
   Accessibility_Accessible retval;
   Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
   AtkObject *child = atk_object_ref_accessible_child (accessible->atko, (gint) index);
-  retval = bonobo_object_corba_objref ( bonobo_object (accessible_new (child)));
+  retval = BONOBO_OBJREF (accessible_new (child));
   printf ("Accessible get_child_at_index called.\n");
+  return CORBA_Object_duplicate (retval, &ev);
+}
+
+/*
+ * CORBA Accessibility::Accessible::getState method implementation
+ */
+static Accessibility_StateSet
+impl_accessibility_accessible_get_state (PortableServer_Servant servant,
+					 CORBA_Environment     *ev)
+{
+  Accessibility_StateSet retval;
+  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  AtkStateSet *state = atk_object_ref_state_set (accessible->atko);
+  retval = CORBA_OBJECT_NIL;
+  printf ("Accessible get_state.\n");
+  /* TODO: implement the bonobo stateset class */
+  return (Accessibility_StateSet) retval;
+}
+
+/*
+ * CORBA Accessibility::Accessible::getRelationSet method implementation
+ */
+static Accessibility_RelationSet *
+impl_accessibility_accessible_get_relation_set (PortableServer_Servant servant,
+						CORBA_Environment     *ev)
+{
+  Accessibility_RelationSet *retval;
+  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  AtkRelationSet *relation_set = atk_object_ref_relation_set (accessible->atko);
+  retval = CORBA_sequence_Accessibility_Relation__alloc ();
+  /*
+   *  TODO: fill the sequence with relation set objects, themselves
+   *  initialized from the AtkRelation object in the AtkRelationSet.
+   */
+  printf ("Accessible get_relation_set.\n");
   return retval;
 }
 
@@ -197,7 +233,6 @@ impl_accessibility_accessible_get_child_at_index (PortableServer_Servant servant
  */
 static Accessibility_Role
 impl_accessibility_accessible_get_role (PortableServer_Servant servant,
-					const CORBA_long      index,
 					CORBA_Environment     *ev)
 {
   Accessibility_Role retval;
@@ -227,8 +262,8 @@ accessible_class_init (AccessibleClass *klass)
         epv->getChildAtIndex = impl_accessibility_accessible_get_child_at_index;
         epv->getIndexInParent = impl_accessibility_accessible_get_index_in_parent;
 
-        /* epv->getRelationSet = impl_accessibility_accessible_get_relation_set;      */
-        /* epv->getState = impl_accessibility_accessible_get_state;                   */
+        epv->getRelationSet = impl_accessibility_accessible_get_relation_set;
+        epv->getState = impl_accessibility_accessible_get_state;
         epv->getRole = impl_accessibility_accessible_get_role;
 }
 
@@ -277,6 +312,8 @@ accessible_new (AtkObject *o)
 {
     Accessible *retval =
                ACCESSIBLE (g_object_new (accessible_get_type (), NULL));
+    CORBA_Environment ev;
+    CORBA_exception_init (&ev);
     g_object_ref (o);
     retval->atko = ATK_OBJECT (o);
 
@@ -317,12 +354,6 @@ accessible_new (AtkObject *o)
                                      BONOBO_OBJECT (text_interface_new (o)));
       }
 
-    if (ATK_IS_HYPERLINK (o))
-      {
-	bonobo_object_add_interface (bonobo_object (retval),
-				     BONOBO_OBJECT (hyperlink_interface_new(o)));
-      }
-
     if (ATK_IS_IMAGE (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
@@ -347,6 +378,5 @@ accessible_new (AtkObject *o)
                                      BONOBO_OBJECT (value_interface_new (o)));
       }
 
-
     return retval;
 }
